#!/usr/bin/env python

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.viptela.vmanage import Vmanage, vmanage_argument_spec
from vmanage.api.device import Device
from vmanage.api.monitor_network import MonitorNetwork
import ipaddress


subset_options = ['config', 'omp', 'control']

def run_module():
    # define available arguments/parameters a user can pass to the module
    argument_spec = vmanage_argument_spec()
    argument_spec.update(device=dict(type='str', aliases=['device', 'host-name']),
                         gather_subset=dict(type='list', options=subset_options),
                         )    

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
        original_message='',
        message=''
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    vmanage = Vmanage(module)
    vmanage_device = Device(vmanage.auth, vmanage.host)
    vmanage_monitor = MonitorNetwork(vmanage.auth, vmanage.host)

    
    if vmanage.params['device']:
        # If we are passed in a device, we return specific facts about that device

        device_facts = {}
        # Check to see if we were passed in a device IP address or a device name
        try:
            system_ip = ipaddress.ip_address(vmanage.params['device'])
            device_status = vmanage_device.get_device_status(system_ip)
        except ValueError:
            device_status = vmanage_device.get_device_status(vmanage.params['device'], key='host-name')
            system_ip = device_status['system-ip']

        if device_status:
            if vmanage.params['gather_subset']:
                requested_subsets = vmanage.params['gather_subset']
            else:
                requested_subsets = subset_options
            device_facts['status'] = device_status
            if 'config' in requested_subsets:
                device_facts['config'] = vmanage_device.get_device_config(device_status['device-type'], system_ip)
            if 'omp' in requested_subsets:
                omp_routes_received = vmanage_monitor.get_omp_routes_received(system_ip)
                omp_routes_advertised = vmanage_monitor.get_omp_routes_advertised(system_ip)
                omp_routes = {
                        'received': omp_routes_received,
                        'advertised': omp_routes_advertised,
                    }
                omp = {
                    'routes': omp_routes
                }
                device_facts['omp'] = omp
            if 'control' in requested_subsets:
                control_connections = vmanage_monitor.get_control_connections(system_ip)
                control_connections_history = vmanage_monitor.get_control_connections_history(system_ip)
                control = {
                    'connections': control_connections,
                    'connections_history': control_connections_history
                }
                device_facts['control'] = control
        vmanage.result['device_facts'] = device_facts
    else:
        if vmanage.params['gather_subset']:
            vmanage.fail_json(msg="gather_subset argument can only be secified with device argument", **vmanage.result)
        # Otherwise, we return facts for all devices sorted by device type
        vmanage.result['vedges'] = vmanage_device.get_device_config_list('vedges')
        vmanage.result['controllers'] = vmanage_device.get_device_config_list('controllers')

    vmanage.exit_json(**vmanage.result)


def main():
    run_module()


if __name__ == '__main__':
    main()



"""Cisco vManage Policy Definitions API Methods.
"""

import json

from vmanage.api.http_methods import HttpMethods
from vmanage.api.policy_lists import PolicyLists
from vmanage.data.parse_methods import ParseMethods
from vmanage.api.utilities import Utilities
from vmanage.utils import list_to_dict

definition_types_19_2_0 = [
    "data", "approute", "control", "cflowd", "mesh", "hubandspoke", "vpnmembershipgroup", "qosmap", "rewriterule",
    "acl", "aclv6", "vedgeroute", "zonebasedfw", "intrusionprevention", "urlfiltering", "advancedMalwareProtection",
    "dnssecurity"
]

definition_types_19_3_0 = [
    "data", "approute", "control", "cflowd", "mesh", "hubandspoke", "vpnmembershipgroup", "qosmap", "rewriterule",
    "acl", "aclv6", "deviceaccesspolicy", "deviceaccesspolicyv6", "vedgeroute", "zonebasedfw", "intrusionprevention",
    "urlfiltering", "advancedMalwareProtection", "dnssecurity", "ssldecryption", "fxoport", "fxsport", "fxsdidport",
    "dialpeer", "srstphoneprofile"
]

all_definition_types = list(set(definition_types_19_2_0) | set(definition_types_19_3_0))


class PolicyDefinitions(object):
    """vManage Policy Definitions API

    Responsible for DELETE, GET, POST, PUT methods against vManage
    Policy Definitions used in Centralized, Localized, and Security Policy.

    """
    def __init__(self, session, host, port=443):
        """Initialize Policy Lists object with session parameters.

        Args:
            session (obj): Requests Session object
            host (str): hostname or IP address of vManage
            port (int): default HTTPS 443

        """

        self.session = session
        self.host = host
        self.port = port
        self.base_url = f'https://{self.host}:{self.port}/dataservice/'
        self.policy_lists = PolicyLists(self.session, self.host, self.port)

        version = Utilities(self.session, self.host, self.port).get_vmanage_version()
        if version >= '19.3.0':
            self.definition_types = definition_types_19_3_0
        else:
            self.definition_types = definition_types_19_2_0

    def get_definition_types(self):
        """Return the definition types for this version of vManage

        Args:

        Returns:
            result (list): List of definition types

        """

        return self.definition_types

    def delete_policy_definition(self, definition_type, definition_id):
        """Delete a Policy Definition from vManage.

        Args:
            definition_type (str): The defintion type of the requested policy definition
            definition_id (str): The defintion ID of the requested policy definition

        Returns:
            result (dict): All data associated with a response.

        """

        url = f"{self.base_url}template/policy/definition/{definition_type.lower()}/{definition_id}"
        HttpMethods(self.session, url).request('DELETE')

    def add_policy_definition(self, policy_definition):
        """Delete a Policy Definition from vManage.

        Args:
            definition_type (str): The defintion type of the requested policy definition
            definition_id (str): The defintion ID of the requested policy definition

        Returns:
            result (dict): All data associated with a response.

        """

        url = f"{self.base_url}template/policy/definition/{policy_definition['type'].lower()}"
        HttpMethods(self.session, url).request('POST', payload=json.dumps(policy_definition))

    def update_policy_definition(self, policy_definition, policy_definition_id):
        """Update a Policy Definition from vManage.

        Args:
            definition_type (str): The defintion type of the requested policy definition
            definition_id (str): The defintion ID of the requested policy definition

        Returns:
            result (dict): All data associated with a response.

        """

        url = f"{self.base_url}template/policy/definition/{policy_definition['type'].lower()}/{policy_definition_id}"
        response = HttpMethods(self.session, url).request('PUT', payload=json.dumps(policy_definition))
        return response

    def get_policy_definition(self, definition_type, definition_id):
        """Get a Policy Definition from vManage.

        Args:
            definition_type (str): The defintion type of the requested policy definition
            definition_id (str): The defintion ID of the requested policy definition

        Returns:
            result (dict): All data associated with a response.

        """

        url = f"{self.base_url}template/policy/definition/{definition_type.lower()}/{definition_id}"
        response = HttpMethods(self.session, url).request('GET')

        policy_definition = response["json"]
        return policy_definition

    def get_policy_definition_list(self, definition_type='all'):
        """Get all Policy Definition Lists from vManage.

        Args:
            definition_type (string): The type of Definition List to retreive

        Returns:
            response (dict): A list of all definition lists currently
                in vManage.

        """

        if definition_type == 'all':
            all_definitions_list = []
            for def_type in self.definition_types:
                definition_list = self.get_policy_definition_list(def_type)
                if definition_list:
                    all_definitions_list.extend(definition_list)
            return all_definitions_list

        definition_list = []

        if definition_type == "advancedMalwareProtection":
            url = f"{self.base_url}template/policy/definition/{definition_type}"
        else:
            url = f"{self.base_url}template/policy/definition/{definition_type.lower()}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        for definition in result:
            definition_detail = self.get_policy_definition(definition_type, definition['definitionId'])
            if definition_detail:
                definition_list.append(definition_detail)
        return definition_list

    def get_policy_definition_dict(self, definition_type, key_name='name', remove_key=False):
        """Get all Policy Definition Lists from vManage.

        Args:
            definition_type (str): Policy definition type
            key_name (string): The name of the attribute to use as the dictionary key
            remove_key (boolean): Remove the search key from the element

        Returns:
            result (dict): All data associated with a response.

        """

        policy_definition_list = self.get_policy_definition_list(definition_type)
        return list_to_dict(policy_definition_list, key_name, remove_key=remove_key)



import click
from vmanage.cli.show.device import device
from vmanage.cli.show.templates import templates
from vmanage.cli.show.policies import policies
from vmanage.cli.show.omp import omp
from vmanage.cli.show.control import control
from vmanage.cli.show.interface import interface
from vmanage.cli.show.route import route
from vmanage.cli.show.org import org
from vmanage.cli.show.vbond import vbond
from vmanage.cli.show.ca_type import ca_type
from vmanage.cli.show.root_cert import root_cert


@click.group()
def show():
    """
    Show commands
    """


show.add_command(device)
show.add_command(templates)
show.add_command(policies)
show.add_command(omp)
show.add_command(control)
show.add_command(interface)
show.add_command(route)
show.add_command(org)
show.add_command(vbond)
show.add_command(ca_type)
show.add_command(root_cert)



import click
from vmanage.api.certificate import Certificate


@click.command('root-cert')
@click.pass_obj
def root_cert(ctx):
    """
    Get vManage root certificate
    """

    vmanage_certificate = Certificate(ctx.auth, ctx.host, ctx.port)
    result = vmanage_certificate.get_vmanage_root_cert()
    click.echo(result)



#!/usr/bin/env python

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}
#### CAN WE DO THIS ????
import os
from ansible.module_utils.basic import AnsibleModule, json
from ansible.module_utils.viptela.viptela import viptelaModule, viptela_argument_spec
from collections import OrderedDict


def run_module():
    # define available arguments/parameters a user can pass to the module
    argument_spec = viptela_argument_spec()
    argument_spec.update(state=dict(type='str', choices=['absent', 'present'], default='present'),
                         file=dict(type='str'),
                         aggregate=dict(type='list')
                         )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    viptela = viptelaModule(module)

    if viptela.params['aggregate']:
        upload_software_list = viptela.params['aggregate']
    else:
        upload_software_list = [
            {
                'file': module.params['file']
            }
        ]

    # THIS MODULE IS DESIGNED TO UPLOAD UPGRADE IMAGES TO THE VMANAGE
    # Software in SD-WAN varies depending on what you want to upgrade.
    # This is a complication for what concern idempotency of this module
    # Files to upgrade vmanage will look like: vmanage-XX.YY.ZZ-<platform>.tar.gz
    # Files to upgrade vedge cloud/vedge 5k/vbond/vsmart will look like: viptela-XX.YY.ZZ-<platform>.tar.gz
    # Physical appliances will NOT have incremental upgrade images
    # CISCO Physical appliances will be upgraded via a new .bin file
    # VIPTELA Physical appliances will be upgraded via a new .tar.gz file

    viptela.result['changed'] = False

    vManage_software_list = viptela.get_software_images_list()

    if viptela.params['state'] == 'present':

        for software_to_upload in upload_software_list:

            try:
                present = False
                path_software_to_be_uploaded = software_to_upload['file']

                if not os.path.exists(path_software_to_be_uploaded):
                    module.fail_json(
                        msg="File does not exists")

                filename_software_to_be_uploaded = os.path.basename(path_software_to_be_uploaded)

                for software in vManage_software_list:
                    availabe_files_list = software["availableFiles"].split(', ')
                    if filename_software_to_be_uploaded in availabe_files_list:
                        present = True

                if not module.check_mode and not present:
                    response = viptela.request('/dataservice/device/action/software/package', method='POST',
                                               files={'file': open(path_software_to_be_uploaded, 'rb')},
                                               data={'validity': 'valid', 'upload': 'true'},
                                               headers=None)

                    viptela.result['changed'] = True
            except Exception as e:
                module.fail_json(
                    msg="General Error {0}".format(e))

    else:
        # absent to be added
        pass

    viptela.exit_json(**viptela.result)


def main():
    run_module()


if __name__ == '__main__':
    main()



"""Cisco vManage Security Policy API Methods.
"""

import json

from vmanage.api.http_methods import HttpMethods
from vmanage.api.policy_definitions import PolicyDefinitions
from vmanage.data.parse_methods import ParseMethods
from vmanage.utils import list_to_dict


class SecurityPolicy(object):
    """vManage Security Policy API

    Responsible for DELETE, GET, POST, PUT methods against vManage
    Security Policy.

    """
    def __init__(self, session, host, port=443):
        """Initialize Security Policy object with session parameters.

        Args:
            session (obj): Requests Session object
            host (str): hostname or IP address of vManage
            port (int): default HTTPS 443

        """

        self.session = session
        self.host = host
        self.port = port
        self.base_url = f'https://{self.host}:{self.port}/dataservice/'
        self.policy_definitions = PolicyDefinitions(self.session, self.host)

    def add_security_policy(self, policy):
        """Add a Security Policy from vManage.

        Args:
            policy: The Security Policy

        Returns:
            result (dict): All data associated with a response.

        """
        url = f"{self.base_url}template/policy/security"
        HttpMethods(self.session, url).request('POST', payload=json.dumps(policy))

    def update_security_policy(self, policy, policy_id):
        """Update a Security from vManage.

        Args:
            policy: The Security Policy
            policy_id: The ID of the Security Policy to update

        Returns:
            result (dict): All data associated with a response.

        """

        url = f"{self.base_url}template/policy/security/{policy_id}"
        HttpMethods(self.session, url).request('PUT', payload=json.dumps(policy))

    def delete_security_policy(self, policyId):
        """Deletes the specified security policy

        Args:
            policyId (str): ID of the active security policy
        Returns:
            result (dict): All data associated with a response.

        """

        api = f"template/policy/security/{policyId}"
        url = self.base_url + api
        response = HttpMethods(self.session, url).request('DELETE')
        result = ParseMethods.parse_status(response)
        return result

    def delete_security_definition(self, definition, definitionId):
        """Deletes the specified policy definition which include:
        'zonebasedfw','urlfiltering', 'dnssecurity','intrusionprevention',
        'advancedMalwareProtection' for 18.4.0 or greater
        and
        'zonebasedfw' for

        Args:
            definition (str): One of the above policy types
            definitionId (str): ID of the policy definitions

        Returns:
            result (dict): All data associated with a response.

        """

        api = f"template/policy/definition/{definition}/{definitionId}"
        url = self.base_url + api
        response = HttpMethods(self.session, url).request('DELETE')
        result = ParseMethods.parse_status(response)
        return result

    def get_security_policy(self):
        """Obtain a list of all configured security policies

        Returns:
            result (dict): All data associated with a response.

        """

        api = "template/policy/security"
        url = self.base_url + api
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_security_definition(self, definition):
        """Obtain a list of various security definitions which include:
        'zonebasedfw','urlfiltering','intrusionprevention',
        'advancedMalwareProtection', 'dnssecurity'

        Args:
            definition (str): One of the above policy types

        Returns:
            result (dict): All data associated with a response.

        """

        api = f"template/policy/definition/{definition}"
        url = self.base_url + api
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_security_policy_list(self):
        """Get all Security Policies from vManage.

        Returns:
            response (list): A list of all policy lists currently
                in vManage.

        """

        security_policy_list = self.get_security_policy()
        # We need to convert the policy definitions from JSON
        for policy in security_policy_list:
            try:
                json_policy = json.loads(policy['policyDefinition'])
                policy['policyDefinition'] = json_policy
            except Exception:  # TODO: figuring out better exception type to catch
                pass
        return security_policy_list

    def get_security_policy_dict(self, key_name='policyName', remove_key=False):
        """Get all Security Policies from vManage.

        Args:
            key_name (str): The name of the attribute to use as the key
            remove_key (bool): Remove the key from the dict (default: False)

        Returns:
            response (dict): A dict of all Security Policies currently
                in vManage.

        """

        security_policy_list = self.get_security_policy_list()

        return list_to_dict(security_policy_list, key_name, remove_key=remove_key)



import click
from vmanage.api.settings import Settings


@click.command()
@click.argument('org_name', nargs=1)
@click.pass_obj
def org(ctx, org_name):
    """
    Set vManage org
    """

    vmanage_settings = Settings(ctx.auth, ctx.host, ctx.port)
    vmanage_settings.set_vmanage_org(org_name)



import ipaddress

import click
from vmanage.api.device import Device
from vmanage.cli.show.print_utils import print_json


@click.command()
@click.argument('device', required=True)
@click.option('--json/--no-json', default=False)
@click.pass_obj
def table(ctx, device, json):
    """
    Show Interfaces
    """
    vmanage_device = Device(ctx.auth, ctx.host, ctx.port)

    # Check to see if we were passed in a device IP address or a device name
    try:
        ip = ipaddress.ip_address(device)
        system_ip = ip
    except ValueError:
        device_dict = vmanage_device.get_device_status(device, key='host-name')
        if 'system-ip' in device_dict:
            system_ip = device_dict['system-ip']
        else:
            system_ip = None

    if not json:
        click.echo("VPNID  PREFIX               NEXT HOP              PROTOCOL      ")
        click.echo("----------------------------------------------------------------")

    routes = vmanage_device.get_device_data('ip/routetable', system_ip)
    for rte in routes:
        if json:
            print_json(rte)
        else:
            if 'nexthop-addr' not in rte:
                rte['nexthop-addr'] = ''
            click.echo(f"{rte['vpn-id']:5}  {rte['prefix']:<20} {rte['nexthop-addr']:<20}  {rte['protocol']:8}")


@click.group()
def route():
    """
    Show device route information
    """


route.add_command(table)



import click
from vmanage.api.central_policy import CentralPolicy
from vmanage.api.utilities import Utilities


@click.command('central-policy')
@click.option('--name', '-n', help="Name of policy to deactivate.")
@click.option('--id', '-i', help="Id of policy to deactivate.")
@click.pass_obj
def central_policy(ctx, name, id):
    """
    deactivate Central Policy
    """

    vmanage_central_policy = CentralPolicy(ctx.auth, ctx.host, ctx.port)
    vmanage_utilities = Utilities(ctx.auth, ctx.host, ctx.port)
    central_policy_dict = vmanage_central_policy.get_central_policy_dict(remove_key=True)
    if id:
        vmanage_central_policy.deactivate_central_policy(id)
    elif name:
        if name in central_policy_dict:
            click.echo(f'Deactivating Central Policy {name}')
            action_id = vmanage_central_policy.deactivate_central_policy(central_policy_dict[name]['policyId'])
            vmanage_utilities.waitfor_action_completion(action_id)
        else:
            click.secho(f'Cannot find Central Policy {name}', fg="red")
    else:
        click.secho('Must specify either policy name of id to deactivate', fg="red")



#!/usr/bin/env python

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

import requests
import os.path
# from requests.auth import HTTPBasicAuth
# from paramiko import SSHClient
# from scp import SCPClient
from ansible.module_utils.basic import AnsibleModule, json
from ansible.module_utils.viptela.viptela import viptelaModule, viptela_argument_spec


def run_module():
    # define available arguments/parameters a user can pass to the module
    argument_spec = viptela_argument_spec()
    argument_spec.update(file=dict(type='str', required=True),
                         )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
        original_message='',
        message=''
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    viptela = viptelaModule(module)

    if not module.check_mode:
        response = viptela.request('/dataservice/system/device/fileupload', method='POST',
                                   files={'file': open(module.params['file'], 'rb')},
                                   data={'validity': 'valid', 'upload': 'true'},
                                   headers=None)

        viptela.result['msg'] = response.json['vedgeListUploadStatus']
        if 'successfully' in response.json['vedgeListUploadStatus']:
            viptela.result['changed'] = True

    viptela.exit_json()


def main():
    run_module()


if __name__ == '__main__':
    main()



"""Cisco vManage Settings API Methods.
"""

import json
from vmanage.api.http_methods import HttpMethods
from vmanage.data.parse_methods import ParseMethods


class Settings(object):
    """vManage Settings API

    Responsible for DELETE, GET, POST, PUT methods against vManage
    Settings.

    """
    def __init__(self, session, host, port=443):
        """Initialize vManage Settings object with session parameters.

        Args:
            session (obj): Requests Session object
            host (str): hostname or IP address of vManage
            port (int): default HTTPS 443

        """

        self.session = session
        self.host = host
        self.port = port
        self.base_url = f'https://{self.host}:{self.port}/dataservice/settings/configuration/'

    def get_vmanage_org(self):
        """Get vManage organization

        Args:

        Returns:
            org (str): The vManage organization.
        """

        url = f"{self.base_url}organization"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        if 'org' in result[0]:
            return result[0]['org']
        return None

    def set_vmanage_org(self, org):
        """Set vManage organization

        Args:
            org (str): The organization name to set.

        Returns:
            result (dict): The result of the POST operation
        """

        payload = {'org': org}
        url = f"{self.base_url}organization"
        response = HttpMethods(self.session, url).request('POST', payload=json.dumps(payload))
        result = ParseMethods.parse_data(response)
        return result[0]

    def get_vmanage_vbond(self):
        """Get vBond

        Args:
            
        Returns:
            result (dict): The result of the GET operation.
        """

        url = f"{self.base_url}device"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        if 'domainIp' in result[0]:
            return result[0]
        return {'domainIp': None, 'port': None}

    def set_vmanage_vbond(self, vbond, vbond_port='12346'):
        """Set vBond

        Args:
            vbond (str): The vBond IP address.
            vbond_port (str): The vBond port.

        Returns:
            result (dict): The result of the POST operation.
        """

        payload = {'domainIp': vbond, 'port': vbond_port}
        url = f"{self.base_url}device"
        response = HttpMethods(self.session, url).request('POST', payload=json.dumps(payload))
        result = ParseMethods.parse_data(response)
        return result[0]

    def get_vmanage_ca_type(self):
        """Get vManage CA type

        Args:

        Returns:
            certificateSigning (str): The CA type.
        """

        url = f"{self.base_url}certificate"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result[0]['certificateSigning']

    def set_vmanage_ca_type(self, ca_type):
        """Set vManage CA type

        Args:
            ca_type (str): The CA type.

        Returns:
            result (dict): The result of the POST operation.
        """

        payload = {'certificateSigning': ca_type, 'challengeAvailable': 'false'}
        url = f"{self.base_url}certificate"
        response = HttpMethods(self.session, url).request('POST', payload=json.dumps(payload))
        result = ParseMethods.parse_data(response)
        return result[0]

    def set_vmanage_root_cert(self, cert, timeout=20):
        """Set vManage root certiticate

        Args:
            cert (str): The root certiticate.
            timeout (int): Timeout in seconds.

        Returns:
        """

        payload = {'enterpriseRootCA': cert}
        url = f"{self.base_url}certificate/enterpriserootca"
        response = HttpMethods(self.session, url).request('PUT', payload=json.dumps(payload), timeout=timeout)
        result = ParseMethods.parse_data(response)
        return result[0]

    def get_vmanage_banner(self):
        """Get vManage Banner

                Args:

                Returns:
                    bannerDetail (str): The banner for vmanage.
                """

        # banner = self.get('settings/configuration/banner')
        # if banner.status_code not in HTTP_SUCCESS_CODES:
        #     return "Error retrieving banner.  Status: {0} Error: {1}".format(banner.status_code, banner.text)
        # return banner.json()['data'][0]['bannerDetail']

        url = f"{self.base_url}banner"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result[0]['bannerDetail']



"""Cisco vManage Monitor Networks API Methods.
"""

from six.moves.urllib.parse import urlencode
from vmanage.api.http_methods import HttpMethods
from vmanage.data.parse_methods import ParseMethods


class MonitorNetwork(object):
    """vManage Monitor Networks API

    Responsible for GET methods against vManage Real Time Monitoring
    for network devices.

    """
    def __init__(self, session, host, port=443):
        """Initialize Monitor Networks object with session parameters.

        Args:
            session (obj): Requests Session object
            host (str): hostname or IP address of vManage
            port (int): default HTTPS 443

        """

        self.session = session
        self.host = host
        self.port = port
        self.base_url = f'https://{self.host}:{self.port}/dataservice/'

    def _get_device_type(self, system_ip):
        device_model = self.get_device_system_info(system_ip)[0]['device-model']
        url = f"{self.base_url}device/models"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        device_type = [device['deviceClass'] for device in result if device_model in device['name']][0]
        return device_type

    def _ipsec_query_params(self, query_params, **kwargs):
        if 'remote_tloc_address' in kwargs:
            query_params.append(('remote-tloc-address', kwargs['remote_tloc_address']))
        if 'remote_tloc_color' in kwargs:
            query_params.append(('remote-tloc-color', kwargs['remote_tloc_color']))
        if 'local_tloc_color' in kwargs:
            query_params.append(('local-tloc-color', kwargs['local_tloc_color']))
        return query_params

    def get_aaa_users(self, system_ip):
        """Provides AAA users for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): Device ARP table data.
        """

        url = f"{self.base_url}device/aaa/users?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_arp_table(self, system_ip):
        """Provides ARP entries for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): Device ARP table data.
        """

        url = f"{self.base_url}device/arp?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bfd_history(self, system_ip, **kwargs):
        """Provides BFD history for device.

        Args:
            system_ip (str): Device System IP
            remote_system_ip (str): (Optional) Remote System IP
            remote_color (str): (Optional) Remote Color

        Returns:
            result (dict): Device BFD history data.
        """

        url = f"{self.base_url}device/bfd/history"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'remote_system_ip' in kwargs:
            query_params.append(('system-ip', kwargs['remote_system_ip']))
        if 'remote_color' in kwargs:
            query_params.append(('color', kwargs['remote_color']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bfd_links(self, system_ip, **kwargs):
        """Provides BFD links for device.

        Args:
            system_ip (str): Device System IP
            state (str): (Optional) State

        Returns:
            result (dict): Device BFD links data.
        """

        url = f"{self.base_url}device/bfd/links"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'state' in kwargs:
            query_params.append(('state', kwargs['state']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bfd_sessions(self, system_ip, **kwargs):
        """Provides BFD sessions for device.

        Args:
            system_ip (str): Device System IP
            remote_system_ip (str): (Optional) Remote System IP
            remote_color (str): (Optional) Remote Color
            local_color (str): (Optional) Local Color

        Returns:
            result (dict): Device BFD sessions data.
        """

        url = f"{self.base_url}device/bfd/sessions"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'remote_system_ip' in kwargs:
            query_params.append(('system-ip', kwargs['remote_system_ip']))
        if 'remote_color' in kwargs:
            query_params.append(('color', kwargs['remote_color']))
        if 'local_color' in kwargs:
            query_params.append(('local-color', kwargs['local_color']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bfd_device_state(self, system_ip):
        """Provides BFD state for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): Device BFD state data.
        """

        url = f"{self.base_url}/device/bfd/state/device?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bfd_device_state_tloc(self, system_ip):
        """Provides BFD state summary with TLOC for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): Device BFD state summary with TLOC data.
        """

        url = f"{self.base_url}/device/bfd/state/device/tloc?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bfd_summary(self, system_ip):
        """Provides BFD summary data for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): Device BFD summary data.
        """

        url = f"{self.base_url}device/bfd/summary?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bfd_tloc(self, system_ip):
        """Provides BFD TLOC for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): Device BFD TLOC data.
        """

        url = f"{self.base_url}device/bfd/tloc?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bgp_neighbors(self, system_ip, **kwargs):
        """Provides BGP neighbors for device.

        Args:
            system_ip (str): Device System IP
            vpn-id (str): VPN ID
            peer-addr (str): Peer address
            as (str): ASN

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/bgp/neighbors"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'vpn_id' in kwargs:
            query_params.append(('vpn-id', kwargs['vpn_id']))
        if 'peer_addr' in kwargs:
            query_params.append(('peer-addr', kwargs['peer_addr']))
        if 'asn' in kwargs:
            query_params.append(('as', kwargs['asn']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_bgp_routes(self, system_ip, **kwargs):
        """Provides BGP routes for device.

        Args:
            system_ip (str): Device System IP
            vpn-id (str): VPN ID
            prefix (str): IP prefix
            nexthop (str): Nexthop

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/bgp/routes"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'vpn_id' in kwargs:
            query_params.append(('vpn-id', kwargs['vpn_id']))
        if 'prefix' in kwargs:
            query_params.append(('prefix', kwargs['prefix']))
        if 'nexthop' in kwargs:
            query_params.append(('nexthop', kwargs['nexthop']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET', timeout=60)
        result = ParseMethods.parse_data(response)
        return result

    def get_bgp_summary(self, system_ip):
        """Provides BGP summary for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/bgp/summary?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_cellular_connections(self, system_ip):
        """Provides celluar connection information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/cellular/connection?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_cellular_hardware(self, system_ip):
        """Provides celluar hardware information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/cellular/hardware?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_cellular_modems(self, policy_id):
        """Provides celluar modem information for device.

        Args:
            policy_id (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/cellular/modem?policyId={policy_id}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_cellular_networks(self, system_ip):
        """Provides celluar network information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/cellular/network?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_cellular_profiles(self, system_ip):
        """Provides celluar profile information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/cellular/profiles?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_cellular_radios(self, system_ip):
        """Provides celluar radio information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/cellular/radio?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_cellular_sessions(self, system_ip, **kwargs):
        """Provides celluar session information for device.

        Args:
            system_ip (str): Device System IP
            interface_name (str): Interface name values: ge0/0 - ge0/7, system, , eth0
            primary_dns_ipv4 (str): Primary DNS IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/cellular/sessions"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'interface_name' in kwargs:
            query_params.append(('if-name', kwargs['interface_name']))
        if 'primary_dns_ipv4' in kwargs:
            query_params.append(('ipv4-dns-pri', kwargs['primary_dns_ipv4']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_cellular_status(self, system_ip):
        """Provides celluar status information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/cellular/status?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_affinity_config(self, system_ip):
        """Provides current control affinity config for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/control/affinity/config?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_affinity_status(self, system_ip):
        """Provides current control affinity status for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/control/affinity/status?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_connections(self, system_ip, **kwargs):
        """Provides current control connections for device.

        Args:
            system_ip (str): Device System IP
            peer_type (str): Peer type (vedge, vsmart, vmanage, vbond)
            peer_system_ip (str): Peer System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/control/connections"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'peer_type' in kwargs:
            query_params.append(('peer-type', kwargs['peer_type']))
        if 'peer_system_ip' in kwargs:
            query_params.append(('system-ip', kwargs['peer_system_ip']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_connections_history(self, system_ip, **kwargs):
        """Provides control connections history for device.

        Args:
            system_ip (str): Device System IP
            peer_type (str): Peer type (vedge, vsmart, vmanage, vbond)
            peer_system_ip (str): Peer System IP
            local_color (str): Local Color

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/control/connectionshistory"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'peer_type' in kwargs:
            query_params.append(('peer-type', kwargs['peer_type']))
        if 'peer_system_ip' in kwargs:
            query_params.append(('system-ip', kwargs['peer_system_ip']))
        if 'local_color' in kwargs:
            query_params.append(('local-color', kwargs['local_color']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_count(self):
        """Provides current control count for device.

        Args:
            None (None):

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/control/count"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_links(self, system_ip, **kwargs):
        """Provides control links for device.

        Args:
            system_ip (str): Device System IP
            state (str): State

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/control/links"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'state' in kwargs:
            query_params.append(('state', kwargs['state']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_local_properties(self, system_ip):
        """Provides control local properties history for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/control/localproperties?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_summary(self, system_ip):
        """Provides current control summary for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/control/summary?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_valid_devices(self, system_ip):
        """Provides control valid devices for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/control/validdevices?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_valid_vsmarts(self, system_ip):
        """Provides control valid vsmarts for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/control/validvsmarts?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_wan_interface(self, system_ip):
        """Provides current control wan interface for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/control/waninterface?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_control_wan_interface_color(self, system_ip):
        """Provides current control wan interface for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/control/waninterface/color?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_device_status(self, system_ip):
        """Provides status for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/system/status?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_device_system_info(self, system_ip):
        """Provides system info for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/system/info?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dhcp_clients(self, system_ip):
        """Provides dhcp clients for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dhcp/client?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dhcp_interfaces(self, system_ip):
        """Provides dhcp interfaces for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}device/dhcp/interfaces?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dhcp_servers(self, system_ip):
        """Provides dhcp servers for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dhcp/server?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dot1x_clients(self, system_ip):
        """Provides DOT1X clients for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dot1x/clients?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dot1x_interfaces(self, system_ip):
        """Provides DOT1X interfaces for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dot1x/interfaces?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dot1x_radius(self, system_ip):
        """Provides DOT1X RADIUS information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dot1x/radius?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dpi_applications(self, system_ip, **kwargs):
        """Provides DPI application information for device.

        Args:
            system_ip (str): Device System IP
            vpn_id (str): VPN/VRF ID
            application (str): Application
            family (str): Family

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dpi/applications"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'vpn_id' in kwargs:
            query_params.append(('vpn-id', kwargs['vpn_id']))
        if 'application' in kwargs:
            query_params.append(('application', kwargs['application']))
        if 'family' in kwargs:
            query_params.append(('family', kwargs['family']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dpi_common_applications(self):
        """Provides common DPI application information.

        Args:
            None (None): None

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dpi/common/applications"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dpi_device_fields(self):
        """Provides DPI fields for device.

        Args:
            None (None): None

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dpi/device/fields"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dpi_device_fields_details(self):
        """Provides DPI detailed fields for device.

        Args:
            None (None): None

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dpi/devicedetails/fields"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dpi_flows(self, system_ip, **kwargs):
        """Provides DPI application information for device.

        Args:
            system_ip (str): Device System IP
            vpn_id (str): VPN/VRF ID
            source_ip (str): Source IP
            application (str): Application
            family (str): Family

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dpi/flows"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'vpn_id' in kwargs:
            query_params.append(('vpn-id', kwargs['vpn_id']))
        if 'source_ip' in kwargs:
            query_params.append(('src-ip', kwargs['source_ip']))
        if 'application' in kwargs:
            query_params.append(('application', kwargs['application']))
        if 'family' in kwargs:
            query_params.append(('family', kwargs['family']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dpi_summary(self, system_ip):
        """Provides DPI summary information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dpi/summary?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_dpi_supported_applications(self, system_ip, **kwargs):
        """Provides DPI supported application information for device.

        Args:
            system_ip (str): Device System IP
            application (str): Application
            family (str): Family

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/dpi/supported-applications"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'application' in kwargs:
            query_params.append(('application', kwargs['application']))
        if 'family' in kwargs:
            query_params.append(('family', kwargs['family']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_eigrp_interfaces(self, system_ip):
        """Provides EIGRP interface for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'cisco-router':
            url = f"{self.base_url}device/eigrp/interface?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_eigrp_routes(self, system_ip):
        """Provides EIGRP route for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'cisco-router':
            url = f"{self.base_url}device/eigrp/route?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET', timeout=60)
        result = ParseMethods.parse_data(response)
        return result

    def get_eigrp_topology(self, system_ip):
        """Provides EIGRP topology for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'cisco-router':
            url = f"{self.base_url}device/eigrp/topology?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_hardware_alarms(self, system_ip):
        """Provides hardware alarms for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/hardware/alarms?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_hardware_environment(self, system_ip):
        """Provides hardware environment information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/hardware/environment?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_hardware_inventory(self, system_ip):
        """Provides hardware inventory information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/hardware/inventory?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_hardware_status_summary(self, system_ip):
        """Provides hardware status summary information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/hardware/status/summary?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_hardware_system(self, system_ip):
        """Provides hardware system information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/hardware/system?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_igmp_groups(self, system_ip):
        """Provides IGMP groups information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/igmp/groups?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_igmp_interfaces(self, system_ip):
        """Provides IGMP interfaces information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/igmp/interface?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_igmp_statistics(self, system_ip):
        """Provides IGMP statistics information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/igmp/statistics?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_igmp_summary(self, system_ip):
        """Provides IGMP summary information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/igmp/summary?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_interface_vpn(self, system_ip):
        """Provides interface vpn/vrf information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/interface/vpn?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ip_fib(self, system_ip, **kwargs):
        """Provides IP FIB for device.

        Args:
            system_ip (str): Device System IP
            vpn_id (str): VPN ID
            address_family (str): Address Family
            destination_prefix (str): Destination Prefix
            tloc (str): TLOC
            color (str): TLOC color

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ip/fib"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'vpn_id' in kwargs:
            query_params.append(('routing-instance-name', kwargs['vpn_id']))
        if 'address_family' in kwargs:
            query_params.append(('rib-address-family', kwargs['address_family']))
        if 'destination_prefix' in kwargs:
            query_params.append(('route-destination-prefix', kwargs['destination_prefix']))
        if 'tloc' in kwargs:
            query_params.append(('tloc', kwargs['tloc']))
        if 'color' in kwargs:
            query_params.append(('color', kwargs['color']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET', timeout=60)
        result = ParseMethods.parse_data(response)
        return result

    def get_ip_route_table(self, system_ip, **kwargs):
        """Provides IP route table for device.

        Args:
            system_ip (str): Device System IP
            vpn_id (str): VPN ID
            address_family (str): Address Family
            destination_prefix (str): Destination Prefix
            source_protocol (str): Source Protocol
            next_hop_address (str): Next-Hop Address (cEdge Only)
            next_hop_oif (str): Next-Hop Outgoing Interface (cEdge Only)

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        query_params = []
        query_params.append(('deviceId', system_ip))
        if device_type == 'cisco-router':
            url = f"{self.base_url}device/ip/ipRoutes"
            if 'vpn_id' in kwargs:
                query_params.append(('routing-instance-name', kwargs['vpn_id']))
            if 'address_family' in kwargs:
                query_params.append(('rib-address-family', kwargs['address_family']))
            if 'destination_prefix' in kwargs:
                query_params.append(('route-destination-prefix', kwargs['destination_prefix']))
            if 'source_protocol' in kwargs:
                query_params.append(('route-source-protocol', kwargs['source_protocol']))
            if 'next_hop_address' in kwargs:
                query_params.append(('next-hop-next-hop-address', kwargs['next_hop_address']))
            if 'next_hop_oif' in kwargs:
                query_params.append(('next-hop-outgoing-interface', kwargs['next_hop_oif']))
        elif device_type == 'viptela-router':
            url = f"{self.base_url}device/ip/routetable"
            if 'vpn_id' in kwargs:
                query_params.append(('vpn-id', kwargs['vpn_id']))
            if 'address_family' in kwargs:
                query_params.append(('address-family', kwargs['address_family']))
            if 'destination_prefix' in kwargs:
                query_params.append(('prefix', kwargs['destination_prefix']))
            if 'source_protocol' in kwargs:
                query_params.append(('protocol', kwargs['source_protocol']))
        else:
            raise Exception(f"Could not retrieve device type {device_type} for {system_ip}")
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET', timeout=60)
        result = ParseMethods.parse_data(response)
        return result

    def get_ip_nat_translations(self, system_ip):
        """Provides IP NAT translations for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ip/nat/translation?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ip_nat64_translations(self, system_ip):
        """Provides IP NAT64 translations for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ip/nat64/translation?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ipsec_inbound_connections(self, system_ip, **kwargs):
        """Provides IPSec inbound connections for device.

        Args:
            system_ip (str): Device System IP
            remote_tloc_address (str): Remote TLOC address
            remote_tloc_color (str): Remote TLOC color
            local_tloc_color (str): Local TLOC color

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ipsec/inbound"
        query_params = []
        query_params.append(('deviceId', system_ip))
        url += '?' + urlencode(self._ipsec_query_params(query_params, **kwargs))
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ipsec_localsa(self, system_ip):
        """Provides IPSec localsa for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ipsec/localsa?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ipsec_outbound_connections(self, system_ip, **kwargs):
        """Provides IPSec outbound connections for device.

        Args:
            system_ip (str): Device System IP
            remote_tloc_address (str): Remote TLOC address
            remote_tloc_color (str): Remote TLOC color

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ipsec/outbound"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'remote_tloc_address' in kwargs:
            query_params.append(('remote-tloc-address', kwargs['remote_tloc_address']))
        if 'remote_tloc_color' in kwargs:
            query_params.append(('remote-tloc-color', kwargs['remote_tloc_color']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ipsec_pwk_inbound_connections(self, system_ip, **kwargs):
        """Provides IPSec pairwise key inbound connections for device.

        Args:
            system_ip (str): Device System IP
            remote_tloc_address (str): Remote TLOC address
            remote_tloc_color (str): Remote TLOC color
            local_tloc_color (str): Local TLOC color

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ipsec/pwk/inbound"
        query_params = []
        query_params.append(('deviceId', system_ip))
        url += '?' + urlencode(self._ipsec_query_params(query_params, **kwargs))
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ipsec_pwk_localsa(self, system_ip, **kwargs):
        """Provides IPSec pairwise key localsa for device.

        Args:
            system_ip (str): Device System IP
            remote_tloc_address (str): Remote TLOC address
            remote_tloc_color (str): Remote TLOC color
            local_tloc_color (str): Local TLOC color

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ipsec/pwk/localsa"
        query_params = []
        query_params.append(('deviceId', system_ip))
        url += '?' + urlencode(self._ipsec_query_params(query_params, **kwargs))
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ipsec_pwk_outbound_connections(self, system_ip, **kwargs):
        """Provides IPSec pairwise key outbound connections for device.

        Args:
            system_ip (str): Device System IP
            remote_tloc_address (str): Remote TLOC address
            remote_tloc_color (str): Remote TLOC color
            local_tloc_color (str): Local TLOC color

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ipsec/pwk/outbound"
        query_params = []
        query_params.append(('deviceId', system_ip))
        url += '?' + urlencode(self._ipsec_query_params(query_params, **kwargs))
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_multicast_replicator(self, system_ip):
        """Provides multicast replicator for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}device/mulitcast/replicator?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_multicast_rpf(self, system_ip):
        """Provides multicast rpf for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}device/mulitcast/rpf?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_multicast_topology(self, system_ip):
        """Provides multicast topology for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}device/mulitcast/topology?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_multicast_tunnel(self, system_ip):
        """Provides multicast tunnel for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}device/mulitcast/tunnel?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_omp_peers(self, system_ip):
        """Provides OMP peers for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/omp/peers?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_omp_routes_received(self, system_ip, **kwargs):
        """Provides OMP received routes for device.

        Args:
            system_ip (str): Device System IP
            vpn_id (str): VPN ID
            prefix (str): Prefix

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/omp/routes/received"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'vpn_id' in kwargs:
            query_params.append(('vpn-id', kwargs['vpn_id']))
        if 'prefix' in kwargs:
            query_params.append(('prefix', kwargs['prefix']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET', timeout=60)
        result = ParseMethods.parse_data(response)
        return result

    def get_omp_routes_advertised(self, system_ip, **kwargs):
        """Provides OMP advertised routes for device.

        Args:
            system_ip (str): Device System IP
            vpn_id (str): VPN ID
            prefix (str): Prefix

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/omp/routes/advertised"
        query_params = []
        query_params.append(('deviceId', system_ip))
        if 'vpn_id' in kwargs:
            query_params.append(('vpn-id', kwargs['vpn_id']))
        if 'prefix' in kwargs:
            query_params.append(('prefix', kwargs['prefix']))
        url += '?' + urlencode(query_params)
        response = HttpMethods(self.session, url).request('GET', timeout=60)
        result = ParseMethods.parse_data(response)
        return result

    def get_omp_summary(self, system_ip):
        """Provides OMP summary for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/omp/summary?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_orchestrator_summary(self, system_ip):
        """Provides orchestrator (vbond) summary for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/orchestrator/summary?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_orchestrator_connections(self, system_ip):
        """Provides orchestrator (vbond) connections for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/orchestrator/connections?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_orchestrator_connections_history(self, system_ip):
        """Provides orchestrator (vbond) connections history for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/orchestrator/connections?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_orchestrator_local_properties(self, system_ip):
        """Provides orchestrator (vbond) local properties history for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}" \
              f"device/orchestrator/localproperties?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ospf_interfaces(self, system_ip):
        """Provides OSPF interfaces for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ospf/interface?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ospf_neighbors(self, system_ip):
        """Provides OSPF neighbors for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/ospf/neighbor?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ospf_routes(self, system_ip):
        """Provides OSPF routes for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}device/ospf/routes?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET', timeout=60)
        result = ParseMethods.parse_data(response)
        return result

    def get_ospf_database(self, system_ip):
        """Provides OSPF database for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}device/ospf/database?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ospf_database_summary(self, system_ip):
        """Provides OSPF database summary for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}" \
              f"device/ospf/databasesummary?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ospf_process(self, system_ip):
        """Provides OSPF process for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}device/ospf/process?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_ospf_database_external(self, system_ip):
        """Provides OSPF database external for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        device_type = self._get_device_type(system_ip)
        if device_type == 'viptela-router':
            url = f"{self.base_url}" \
              f"device/ospf/databaseexternal?deviceId={system_ip}"
        else:
            raise Exception(f"Device type {device_type} for {system_ip} is not valid for this API endpoint.")
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_pim_interfaces(self, system_ip):
        """Provides PIM interfaces for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/pim/interface?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_pim_neighbors(self, system_ip):
        """Provides PIM neighbors for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/pim/neighbor?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_pim_rp_mapping(self, system_ip):
        """Provides PIM RP mapping for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/pim/rp-mapping?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_pim_statistics(self, system_ip):
        """Provides PIM statistics for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/pim/statistics?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_security_information(self, system_ip):
        """Provides security information for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/security/information?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_software(self, system_ip):
        """Provides software for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/software?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_umbrella_sig_tunnels(self, system_ip):
        """Provides Cisco Umbrella SIG tunnels for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/sig/umbrella/tunnels?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_users(self, system_ip):
        """Provides users for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): Device ARP table data.
        """

        url = f"{self.base_url}device/users?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_users_list(self, system_ip):
        """Provides users list for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): Device ARP table data.
        """

        url = f"{self.base_url}device/users/list?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_vrrp(self, system_ip):
        """Provides VRRP entries for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/vrrp?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_wlan_clients(self, system_ip):
        """Provides WLAN clients for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/wlan/clients?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_wlan_interfaces(self, system_ip):
        """Provides WLAN interfaces for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/wlan/interfaces?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_wlan_radios(self, system_ip):
        """Provides WLAN radios for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/wlan/radios?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_wlan_radius(self, system_ip):
        """Provides WLAN RADIUS authenticiation for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/wlan/radius?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result

    def get_zscaler_sig_tunnels(self, system_ip):
        """Provides Cisco zScaler SIG tunnels for device.

        Args:
            system_ip (str): Device System IP

        Returns:
            result (dict): All data associated with a response.
        """

        url = f"{self.base_url}device/sig/zscaler/tunnels?deviceId={system_ip}"
        response = HttpMethods(self.session, url).request('GET')
        result = ParseMethods.parse_data(response)
        return result



#!/usr/bin/env python

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

import time
from ansible.module_utils.basic import AnsibleModule, json
from ansible.module_utils.viptela.viptela import viptelaModule, viptela_argument_spec


def run_module():
    action_id = None
    action_status = None
    action_activity = None
    action_config = None

    # define available arguments/parameters a user can pass to the module
    argument_spec = viptela_argument_spec()
    argument_spec.update(state=dict(type='str', choices=['absent', 'present', 'query'], default='present'),
                         device_name=dict(type='str', aliases=['device', 'host-name']),
                         device_ip=dict(type='str', aliases=['system_ip']),
                         site_id=dict(type='str'),
                         uuid=dict(type='str'),
                         personality=dict(type='str', choices=['vmanage', 'vsmart', 'vbond', 'vedge'], default='vedge'),
                         template=dict(type='str'),
                         variables=dict(type='dict', default={}),
                         wait=dict(type='bool', default=False),
                         )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    viptela = viptelaModule(module)
    viptela.result['what_changed'] = []

    if viptela.params['personality'] == 'vedge':
        device_type = 'vedges'
    else:
        device_type = 'controllers'

    if viptela.params['uuid']:
        device_data = viptela.get_device_by_uuid(viptela.params['uuid'], type=device_type)
        if 'uuid' not in device_data:
            viptela.fail_json(msg='Cannot find device with UUID: {0}.'.format(viptela.params['uuid']))
        # If this is a preallocation, we need to set these things.
        if 'system-ip' not in device_data or len(device_data['system-ip']) == 0:
            if viptela.params['system_ip']:
                device_data['system-ip'] = viptela.params['system_ip']
            else:
                viptela.fail_json(msg='system_ip is needed when pre-attaching templates')
        if 'deviceIP' not in device_data or len(device_data['deviceIP']) == 0:
            if viptela.params['system_ip']:
                device_data['deviceIP'] = viptela.params['system_ip']
            else:
                viptela.fail_json(msg='system_ip is needed when pre-attaching templates')
        if 'site-id' not in device_data or len(device_data['site-id']) == 0:
            if viptela.params['site_id']:
                device_data['site-id'] = viptela.params['site_id']
            else:
                viptela.fail_json(msg='site_id is needed when pre-attaching templates')
        if 'host-name' not in device_data or len(device_data['host-name']) == 0:
            if viptela.params['device_name']:
                device_data['host-name'] = viptela.params['device_name']
            else:
                viptela.fail_json(msg='device_name is needed when pre-attaching templates')
    elif viptela.params['device_name']:
        device_status = viptela.get_device_status(viptela.params['device_name'], key='host-name')
        if 'uuid' in device_status:
            device_type = 'controllers' if device_status['personality'] in ['vmanage', 'vbond', 'vsmart'] else 'vedges'
            device_data = viptela.get_device_by_uuid(device_status['uuid'], type=device_type)
        else:
            viptela.fail_json(msg='Cannot find device with name: {0}.'.format(viptela.params['device']))

    if viptela.params['state'] == 'present':
        if 'system-ip' not in device_data:
            viptela.fail_json(msg='system-ip must be defined for {0}.'.format(viptela.params['device']))
        if 'site-id' not in device_data:
            viptela.fail_json(msg='site-id must be defined for {0}.'.format(viptela.params['device']))

        # Get template data and see if it is a real template
        device_template_dict = viptela.get_device_template_dict(factory_default=True)
        if viptela.params['template']:
            if viptela.params['template'] not in device_template_dict:
                viptela.fail_json(msg='Template {0} not found.'.format(viptela.params['template']))
            template_data = device_template_dict[viptela.params['template']]
        else:
            viptela.fail_json(msg='Must specify a template with state present')

        # Make sure they passed in the required variables
        # get_template_variables provides a variable name -> property mapping
        template_variables = viptela.get_template_variables(device_template_dict[viptela.params['template']]['templateId'])
        optional_template_variables = viptela.get_template_optional_variables(
            device_template_dict[viptela.params['template']]['templateId'])
        mandatory_template_variables = {k: template_variables[k] for k in set(template_variables) - set(optional_template_variables)}
        if mandatory_template_variables:
            if viptela.params['variables']:
                for variable in mandatory_template_variables:
                    if variable not in viptela.params['variables']:
                        viptela.fail_json(
                            msg='Template {0} requires variables: {1}'.format(viptela.params['template'], ', '.join(template_variables)))

        viptela.result['template_variables'] = template_variables

        # Construct the variable payload
        device_template_variables = {
            "csv-status": "complete",
            "csv-deviceId": device_data['uuid'],
            "csv-deviceIP": device_data['deviceIP'],
            "csv-host-name": device_data['host-name'],
            '//system/host-name': device_data['host-name'],
            '//system/system-ip': device_data['system-ip'],
            '//system/site-id': device_data['site-id'],
        }

        # For each of the variables passed in, match them up with the names of the variables requires in the
        # templates and add them with the corresponding property.  The the variables is not in template_variables,
        # just leave it out since it is not required.
        for key, value in viptela.params['variables'].items():
            if key in template_variables:
                property_value = template_variables[key]
                device_template_variables[property_value] = viptela.params['variables'][key]

        # When dealing with optional parameters if we do not have explicitely set a value for it
        # we must add the optional parameter to the payload with { key: 'TEMPLATE_IGNORE'}
        for key, value in optional_template_variables.items():
            property_value = template_variables[key]
            if property_value not in device_template_variables:
                device_template_variables[property_value] = 'TEMPLATE_IGNORE'

        attached_uuid_list = viptela.get_template_attachments(template_data['templateId'], key='uuid')

        if device_data['uuid'] in attached_uuid_list:
            # Add the template ID to the device's variable payload because we'll need it for comparison and update.
            # device_template_variables['csv-templateId'] = template_data['templateId']
            # The device is already attached to the template.  We need to see if any of the input changed, so we make
            # an API call to get the input on last attach
            payload = {
                "templateId": template_data['templateId'],
                "deviceIds": [device_data['uuid']],
                "isEdited": "true",
                "isMasterEdited": "false"
            }
            response = viptela.request('/dataservice/template/device/config/input/', method='POST', payload=payload)
            if response.json and 'data' in response.json:
                current_variables = response.json['data'][0]
                # viptela.result['old'] = current_variables
                # viptela.result['new'] = device_template_variables
                # Convert both to a string and compare.  For some reason, there can be an int/str
                # mismatch.  It might be indicative of a problem...
                for property in device_template_variables:
                    if str(device_template_variables[property]) != str(current_variables[property]):
                        viptela.result['changed'] = True
        else:
            viptela.result['changed'] = True

        if not module.check_mode and viptela.result['changed']:
            payload = {
                "deviceTemplateList":
                    [
                        {
                            "templateId": template_data['templateId'],
                            "device": [device_template_variables],
                            "isEdited": False,
                            "isMasterEdited": False
                        }
                    ]
            }
            response = viptela.request('/dataservice/template/device/config/attachfeature', method='POST', payload=payload)
            if response.json:
                action_id = response.json['id']
            else:
                viptela.fail_json(msg='Did not get action ID after attaching device to template.')
    elif viptela.params['state'] == 'absent':
        if 'templateId' in device_data:
            viptela.result['changed'] = True
            payload = {
                "deviceType": device_data['deviceType'],
                "devices": [
                    {
                        "deviceId": device_data['uuid'],
                        "deviceIP": device_data['deviceIP']
                    }
                ]
            }
            if not module.check_mode:
                response = viptela.request('/dataservice/template/config/device/mode/cli', method='POST', payload=payload)
                if response.json:
                    action_id = response.json['id']
                else:
                    viptela.fail_json(msg='Did not get action ID after attaching device to template.')

    elif viptela.params['state'] == 'query':
        # Get template data and see if it is a real template
        device_template_dict = viptela.get_device_template_dict(factory_default=True)
        if viptela.params['template']:
            if viptela.params['template'] not in device_template_dict:
                viptela.fail_json(msg='Template {0} not found.'.format(viptela.params['template']))
            template_data = device_template_dict[viptela.params['template']]
        else:
            viptela.fail_json(msg='Must specify a template with state query')

        # get_template_variables provides a variable name -> property mapping
        template_variables = viptela.get_template_variables(device_template_dict[viptela.params['template']]['templateId'])
        optional_template_variables = viptela.get_template_optional_variables(
            device_template_dict[viptela.params['template']]['templateId'])
        viptela.result['template_variables'] = template_variables
        viptela.result['optional_template_variables'] = optional_template_variables
        viptela.result['mandatory_template_variables'] = {k: template_variables[k] for k in
                                                          set(template_variables) - set(optional_template_variables)}
    # If told, wait for the status of the request and report it
    if viptela.params['wait'] and action_id:
        viptela.waitfor_action_completion(action_id)

    viptela.logout()
    viptela.exit_json(**viptela.result)


def main():
    run_module()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

from ansible.module_utils.basic import AnsibleModule, json
from ansible.module_utils.viptela.viptela import viptelaModule, viptela_argument_spec
from collections import OrderedDict


def run_module():
    # define available arguments/parameters a user can pass to the module
    argument_spec = viptela_argument_spec()
    argument_spec.update(devices=dict(type='list'),
                         deviceType=dict(type='str', choices=['controller', 'vedge'], default='vedge'),
                         version=dict(type='str'),
                         activate=dict(type='bool'),
                         set_default=dict(type='bool')
                         )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    viptela = viptelaModule(module)

    # It seems to me that the software operations on the vManage are already idempotent
    # This is why at least in this phase we are not implementing other idempotency checks
    # In future it might make sense to implement them to avoid generating useless tasks
    # on the vManage. This would streamline operations.
    # TODO Add idempotency cheks to this module

    devices = module.params['devices']
    deviceType = module.params['deviceType']
    version = module.params['version']
    data = [
        {
            "family": "vedge-x86",
            "version": version
        }
    ]
    reboot = module.params['activate']
    set_default = module.params['set_default']

    # Verify if the target software (version) is available in the vManage and set flag
    vManage_software_list = viptela.get_software_images_list()
    software_present_on_vManage = False
    for software in vManage_software_list:
        if version == software['versionName']:
            software_present_on_vManage = True

    # If we have found the software we can move on and perform the operation
    if software_present_on_vManage:
        viptela.software_install(devices, deviceType, data, reboot)

        if set_default:
            for device in devices:
                device.update({'version': version})
            viptela.set_default_partition(devices, deviceType)

    # If not, we fail
    else:
        module.fail_json(
            msg="We don't have this software, I am sorry")

    viptela.exit_json(**viptela.result)


def main():
    run_module()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.viptela.vmanage import Vmanage, vmanage_argument_spec
from vmanage.api.certificate import Certificate

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

def run_module():
    # define available arguments/parameters a user can pass to the module
    argument_spec = vmanage_argument_spec()
    argument_spec.update(organization=dict(type='str'),
                         vbond=dict(type='str'),
                         vbond_port=dict(type='int', default=12346),
                         root_cert=dict(type='str'),
                         push=dict(type='bool')
                         )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    vmanage = Vmanage(module)
    vmanage_certificate = Certificate(vmanage.auth, vmanage.host)

    vmanage.result['what_changed'] = []

    if vmanage.params['push']:
        vmanage_certificate.push_certificates()

    if vmanage.result['what_changed']:
        vmanage.result['changed'] = True

    vmanage.exit_json(**vmanage.result)


def main():
    run_module()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.viptela.vmanage import Vmanage, vmanage_argument_spec
from vmanage.api.settings import Settings
from vmanage.api.certificate import Certificate

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

def run_module():
    # define available arguments/parameters a user can pass to the module
    argument_spec = vmanage_argument_spec()
    argument_spec.update(organization=dict(type='str'),
                         vbond=dict(type='str'),
                         vbond_port=dict(type='str', default='12346'),
                         root_cert=dict(type='str'),
                         ca_type=dict(type='str')
                         )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    vmanage = Vmanage(module)
    vmanage_settings = Settings(vmanage.auth, vmanage.host)
    vmanage_certificate = Certificate(vmanage.auth, vmanage.host)
    vmanage.result['what_changed'] = []

    if vmanage.params['organization']:
        current = vmanage_settings.get_vmanage_org()
        if vmanage.params['organization'] != current:
            vmanage.result['what_changed'].append('organization')
            if not module.check_mode:
                vmanage_settings.set_vmanage_org(vmanage.params['organization'])

    if vmanage.params['vbond']:
        current = vmanage_settings.get_vmanage_vbond()
        if vmanage.params['vbond'] != current['domainIp'] or vmanage.params['vbond_port'] != current['port']:
            vmanage.result['what_changed'].append('vbond')
            if not module.check_mode:
                vmanage_settings.set_vmanage_vbond(vmanage.params['vbond'], vmanage.params['vbond_port'])

    if vmanage.params['ca_type']:
        current = vmanage_settings.get_vmanage_ca_type()
        if vmanage.params['ca_type'] != current:
            vmanage.result['what_changed'].append('ca_type')
            if not module.check_mode:
                vmanage_settings.set_vmanage_ca_type(vmanage.params['ca_type'])

    if vmanage.params['root_cert']:
        current = vmanage_certificate.get_vmanage_root_cert()
        if vmanage.params['root_cert'] not in current:
            vmanage.result['what_changed'].append('root_cert')
            if not module.check_mode:
                vmanage_settings.set_vmanage_root_cert(vmanage.params['root_cert'])

    if vmanage.result['what_changed']:
        vmanage.result['changed'] = True

    vmanage.exit_json(**vmanage.result)


def main():
    run_module()


if __name__ == '__main__':
    main()



import click
from vmanage.api.certificate import Certificate


@click.command()
@click.pass_obj
def push(ctx):
    """
    Push certificates to all controllers
    """

    vmanage_certificate = Certificate(ctx.auth, ctx.host, ctx.port)
    click.echo("Pushing certificates to controllers...")
    vmanage_certificate.push_certificates()



def list_to_dict(lst, key_name, remove_key=True):
    """Convert a list of dictionaries into a dictionary of dictionaries.

    Args:
        key_name: The name of the key from the child dictionary to use to use in the parent.
        remove_key: Remove the key used for the parent dictionary from the child.


    Returns:
        result (dict): The resulting dictionary.

    """
    d = {}
    for item in lst:
        if key_name in item:
            if remove_key:
                key = item.pop(key_name)
            else:
                key = item[key_name]

            d[key] = item

    return d



import ipaddress

import click
from vmanage.api.device import Device
from vmanage.cli.show.print_utils import print_json


@click.command(name='list')
@click.argument('device', required=True)
@click.option('--json/--no-json', default=False)
@click.pass_obj
def list_interface(ctx, device, json):
    """
    Show Interfaces
    """
    vmanage_device = Device(ctx.auth, ctx.host, ctx.port)

    if device:
        # Check to see if we were passed in a device IP address or a device name
        try:
            ipaddress.ip_address(device)
            system_ip = device
        except ValueError:
            device_dict = vmanage_device.get_device_status(device, key='host-name')
            if 'system-ip' in device_dict:
                system_ip = device_dict['system-ip']
        device_list = [system_ip]

    if not json:
        click.echo("IFNAME            VPNID  IP ADDR          MAC ADDR                  OPER STATE            DESC")
        click.echo("-" * 118)

    for dev in device_list:
        interfaces = vmanage_device.get_device_data('interface', dev)
        for iface in interfaces:
            if json:
                print_json(iface)
            else:
                if 'hwaddr' not in iface:
                    iface['hwaddr'] = ''
                if 'desc' not in iface:
                    iface['desc'] = ''
                if 'ip-address' not in iface:
                    iface['ip-address'] = ''
                click.echo(
                    f"{iface['ifname']:17} {iface['vpn-id']:6} {iface['ip-address']:16} {iface['hwaddr']:25} {iface['if-oper-status']:17} {iface['desc']:17}"
                )


@click.group()
def interface():
    """
    Show real-time information
    """


interface.add_command(list_interface)



import click
from vmanage.apps.files import Files


@click.command()
@click.option('--type',
              '-t',
              'device_type',
              help="Device Type",
              type=click.Choice(['controllers', 'vedges']),
              default=None)
@click.option('--name', '-n', multiple=True)
@click.option('--file', '-f', 'output_file', help="Output file name", required=True)
@click.pass_obj
def attachments(ctx, device_type, name, output_file):
    """
    Export attachments to file
    """
    num = 0
    vmanage_files = Files(ctx.auth, ctx.host, ctx.port)
    if device_type == 'controllers':
        if name:
            click.echo(f'Exporting controller attachment(s) {",".join(name)} to {output_file}')
            num = vmanage_files.export_attachments_to_file(output_file, name_list=name, device_type=device_type)
        else:
            click.echo(f'Exporting controller attachment to {output_file}')
            num = vmanage_files.export_attachments_to_file(output_file, device_type=device_type)
    elif device_type == 'vedges':
        if name:
            click.echo(f'Exporting vedge attachment(s) {",".join(name)} to {output_file}')
            num = vmanage_files.export_attachments_to_file(output_file, name_list=name, device_type=device_type)
        else:
            click.echo(f'Exporting vedge attachment to {output_file}')
            num = vmanage_files.export_attachments_to_file(output_file, device_type=device_type)
    else:
        if name:
            click.echo(f'Exporting attachment(s) {",".join(name)} to {output_file}')
            num = vmanage_files.export_attachments_to_file(output_file, name_list=name, device_type=device_type)
        else:
            click.echo(f'Exporting attachment to {output_file}')
            num = vmanage_files.export_attachments_to_file(output_file, device_type=device_type)
    click.echo(f"Exported {num} attachments")



import click
from vmanage.api.settings import Settings


@click.command('ca-type')
@click.option('--type', 'type_', required=True, type=click.Choice(['enterprise'], case_sensitive=False))
@click.pass_obj
def ca_type(ctx, type_):
    """
    Set vManage CA type
    """

    vmanage_settings = Settings(ctx.auth, ctx.host, ctx.port)
    vmanage_settings.set_vmanage_ca_type(type_)



import click
from vmanage.apps.files import Files


@click.command()
# @click.argument(policy_type, default='all')
@click.option('--file', '-f', 'export_file', help="output File name", required=True)
# @click.option('--type',
#               help="Device type [vedges, controllers]",
#               type=click.Choice(['vedges', 'controllers']))
@click.pass_obj
def policies(ctx, export_file):
    """
    Export policies to file
    """

    vmanage_files = Files(ctx.auth, ctx.host, ctx.port)
    click.echo(f'Exporting policies to {export_file}')
    vmanage_files.export_policy_to_file(export_file)



import click
from vmanage.api.settings import Settings


@click.command()
@click.option('--ip', 'ip', help="IP address of the vBond", required=True)
@click.option('--port', 'port', help="Port of the vBond", default='12346', required=False)
@click.pass_obj
def vbond(ctx, ip, port):
    """
    Set vBond
    """

    vmanage_settings = Settings(ctx.auth, ctx.host, ctx.port)
    vmanage_settings.set_vmanage_vbond(ip, port)



from os import path
from setuptools import setup, find_namespace_packages

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

includes = [
    "vmanage",
    "vmanage.*",
    "ansible.modules.viptela",
    "ansible.module_utils.viptela",
    "ansible.plugins.httpapi",
]

setup(
    name="viptela",
    version='0.3.9',
    packages=find_namespace_packages(include=includes),
    description="Cisco DevNet SD-WAN vManage (Viptela) CLI/SDK",
    install_requires=['Click', 'requests', 'dictdiffer', 'PyYAML'],
    entry_points='''
        [console_scripts]
        vmanage=vmanage.__main__:vmanage
    ''',
    long_description=long_description,
    long_description_content_type='text/markdown',
)



"""Cisco vManage Files API Methods.
"""

import json
import os
import yaml
from vmanage.api.device_templates import DeviceTemplates
from vmanage.api.feature_templates import FeatureTemplates
from vmanage.data.template_data import TemplateData
from vmanage.data.policy_data import PolicyData
from vmanage.api.policy_lists import PolicyLists
from vmanage.api.policy_definitions import PolicyDefinitions
from vmanage.api.local_policy import LocalPolicy
from vmanage.api.central_policy import CentralPolicy
from vmanage.api.security_policy import SecurityPolicy
from vmanage.api.device import Device
from vmanage.utils import list_to_dict


class Files(object):
    """Read and write data to file.


    """
    def __init__(self, session, host, port=443):
        """Initialize Files object with session parameters.

        Args:
            session (obj): Requests Session object
            host (str): hostname or IP address of vManage
            port (int): default HTTPS 443

        """

        self.session = session
        self.host = host
        self.port = port
        self.base_url = f'https://{self.host}:{self.port}/dataservice/'
        self.device_templates = DeviceTemplates(self.session, self.host, self.port)
        self.feature_templates = FeatureTemplates(self.session, self.host, self.port)
        self.template_data = TemplateData(self.session, self.host, self.port)
        self.policy_data = PolicyData(self.session, self.host, self.port)
        self.policy_lists = PolicyLists(self.session, self.host, self.port)
        self.policy_definitions = PolicyDefinitions(self.session, self.host, self.port)
        self.local_policy = LocalPolicy(self.session, self.host, self.port)
        self.central_policy = CentralPolicy(self.session, self.host, self.port)
        self.security_policy = SecurityPolicy(self.session, self.host, self.port)
        self.vmanage_device = Device(self.session, self.host, self.port)

    def export_templates_to_file(self, export_file, name_list=None, template_type=None):
        """Export templates to a file.  All object IDs will be translated to names.  Use
        a '.yml' extention to export as YAML and a '.json' extension to export as JSON.

        Args:
            export_file (str): The name of the export file
            name_list (list): List of device templates to export
            template_type (str): Template type: device or template

        """

        template_export = {}
        #pylint: disable=too-many-nested-blocks
        if template_type != 'feature':
            # Export the device templates and associated feature templates
            device_template_list = self.template_data.export_device_template_list(name_list=name_list)
            template_export.update({'vmanage_device_templates': device_template_list})
            feature_name_list = []
            if name_list:
                for device_template in device_template_list:
                    if 'generalTemplates' in device_template:
                        for general_template in device_template['generalTemplates']:
                            if 'templateName' in general_template:
                                feature_name_list.append(general_template['templateName'])
                            if 'subTemplates' in general_template:
                                for sub_template in general_template['subTemplates']:
                                    if 'templateName' in sub_template:
                                        feature_name_list.append(sub_template['templateName'])
                name_list = list(set(feature_name_list))
        # Since device templates depend on feature templates, we always add them.
        feature_template_list = self.feature_templates.get_feature_template_list(name_list=name_list)
        template_export.update({'vmanage_feature_templates': feature_template_list})

        if export_file.endswith('.json'):
            with open(export_file, 'w') as outfile:
                json.dump(template_export, outfile, indent=4, sort_keys=False)
        elif export_file.endswith('.yaml') or export_file.endswith('.yml'):
            with open(export_file, 'w') as outfile:
                yaml.dump(template_export, outfile, indent=4, sort_keys=False)
        else:
            raise Exception("File format not supported")

    #pylint: disable=unused-argument
    def import_templates_from_file(self,
                                   import_file,
                                   update=False,
                                   check_mode=False,
                                   name_list=None,
                                   template_type=None):
        """Import templates from a file.  All object Names will be translated to IDs.

        Args:
            import_file (str): The name of the import file
            name_list (list): List of device templates to export
            template_type (str): Template type: device or template
            check_mode (bool): Try the import, but don't make changes (default: False)
            update (bool): Update existing templates (default: False)

        """
        feature_template_updates = []
        device_template_updates = []
        imported_template_data = {}

        # Read in the datafile
        if not os.path.exists(import_file):
            raise Exception(f"Cannot find file {import_file}")
        with open(import_file) as f:
            if import_file.endswith('.yaml') or import_file.endswith('.yml'):
                imported_template_data = yaml.safe_load(f)
            else:
                imported_template_data = json.load(f)

        if 'vmanage_feature_templates' in imported_template_data:
            imported_feature_template_list = imported_template_data['vmanage_feature_templates']
        else:
            imported_feature_template_list = []

        imported_device_template_list = []

        #pylint: disable=too-many-nested-blocks
        if template_type != 'feature':
            # Import the device templates and associated feature templates
            if 'vmanage_device_templates' in imported_template_data:
                imported_device_template_list = imported_template_data['vmanage_device_templates']
            if name_list:
                feature_name_list = []
                pruned_device_template_list = []
                for device_template in imported_device_template_list:
                    if device_template['templateName'] in name_list:
                        pruned_device_template_list.append(device_template)
                        if 'generalTemplates' in device_template:
                            for general_template in device_template['generalTemplates']:
                                if 'templateName' in general_template:
                                    feature_name_list.append(general_template['templateName'])
                                if 'subTemplates' in general_template:
                                    for sub_template in general_template['subTemplates']:
                                        if 'templateName' in sub_template:
                                            feature_name_list.append(sub_template['templateName'])
                imported_device_template_list = pruned_device_template_list
                name_list = list(set(feature_name_list))
        # Since device templates depend on feature templates, we always add them.
        if name_list:
            pruned_feature_template_list = []
            imported_feature_template_dict = list_to_dict(imported_feature_template_list,
                                                          key_name='templateName',
                                                          remove_key=False)
            for feature_template_name in name_list:
                if feature_template_name in imported_feature_template_dict:
                    pruned_feature_template_list.append(imported_feature_template_dict[feature_template_name])
                # Otherwise, we hope the feature list is already there (e.g. Factory Default)
            imported_feature_template_list = pruned_feature_template_list

        # Process the feature templates
        feature_template_updates = self.template_data.import_feature_template_list(imported_feature_template_list,
                                                                                   check_mode=check_mode,
                                                                                   update=update)

        # Process the device templates
        device_template_updates = self.template_data.import_device_template_list(imported_device_template_list,
                                                                                 check_mode=check_mode,
                                                                                 update=update)

        return {
            'feature_template_updates': feature_template_updates,
            'device_template_updates': device_template_updates,
        }

    #
    # Policy
    #
    def export_policy_to_file(self, export_file):
        """Export policy to a file.  All object IDs will be translated to names.  Use
        a '.yml' extention to export as YAML and a '.json' extension to export as JSON.

        Args:
            export_file (str): The name of the export file

        """

        policy_lists_list = self.policy_lists.get_policy_list_list()
        policy_definitions_list = self.policy_data.export_policy_definition_list()
        central_policies_list = self.policy_data.export_central_policy_list()
        local_policies_list = self.local_policy.get_local_policy_list()
        security_policies_list = self.policy_data.export_security_policy_list()

        policy_export = {
            'vmanage_policy_lists': policy_lists_list,
            'vmanage_policy_definitions': policy_definitions_list,
            'vmanage_central_policies': central_policies_list,
            'vmanage_local_policies': local_policies_list,
            'vmanage_security_policies': security_policies_list
        }

        if export_file.endswith('.json'):
            with open(export_file, 'w') as outfile:
                json.dump(policy_export, outfile, indent=4, sort_keys=False)
        elif export_file.endswith(('.yaml', 'yml')):
            with open(export_file, 'w') as outfile:
                yaml.dump(policy_export, outfile, default_flow_style=False)
        else:
            raise Exception("File format not supported")

    def import_policy_from_file(self, file, update=False, check_mode=False, push=False):
        """Import policy from a file.  All object Names will be translated to IDs.

        Args:
            import_file (str): The name of the import file
            check_mode (bool): Try the import, but don't make changes (default: False)
            update (bool): Update existing templates (default: False)
            push (bool): Push tempaltes to devices if changed (default: False)

        """

        policy_list_updates = []
        policy_definition_updates = []
        central_policy_updates = []
        local_policy_updates = []
        security_policy_updates = []

        # Read in the datafile
        if not os.path.exists(file):
            raise Exception('Cannot find file {0}'.format(file))
        with open(file) as f:
            if file.endswith('.yaml') or file.endswith('.yml'):
                policy_data = yaml.safe_load(f)
            else:
                policy_data = json.load(f)

        # Separate the feature template data from the device template data
        if 'vmanage_policy_lists' in policy_data:
            policy_list_data = policy_data['vmanage_policy_lists']
        else:
            policy_list_data = []
        if 'vmanage_policy_definitions' in policy_data:
            policy_definition_data = policy_data['vmanage_policy_definitions']
        else:
            policy_definition_data = []
        if 'vmanage_central_policies' in policy_data:
            central_policy_data = policy_data['vmanage_central_policies']
        else:
            central_policy_data = []
        if 'vmanage_local_policies' in policy_data:
            local_policy_data = policy_data['vmanage_local_policies']
        else:
            local_policy_data = []
        if 'vmanage_security_policies' in policy_data:
            security_policy_data = policy_data['vmanage_security_policies']
        else:
            security_policy_data = []

        policy_list_updates = self.policy_data.import_policy_list_list(policy_list_data,
                                                                       check_mode=check_mode,
                                                                       update=update,
                                                                       push=push)

        self.policy_lists.clear_policy_list_cache()

        policy_definition_updates = self.policy_data.import_policy_definition_list(policy_definition_data,
                                                                                   check_mode=check_mode,
                                                                                   update=update,
                                                                                   push=push)
        central_policy_updates = self.policy_data.import_central_policy_list(central_policy_data,
                                                                             check_mode=check_mode,
                                                                             update=update,
                                                                             push=push)
        local_policy_updates = self.policy_data.import_local_policy_list(local_policy_data,
                                                                         check_mode=check_mode,
                                                                         update=update,
                                                                         push=push)
        security_policy_updates = self.policy_data.import_security_policy_list(security_policy_data,
                                                                               check_mode=check_mode,
                                                                               update=update,
                                                                               push=push)

        return {
            'policy_list_updates': policy_list_updates,
            'policy_definition_updates': policy_definition_updates,
            'central_policy_updates': central_policy_updates,
            'local_policy_updates': local_policy_updates,
            'security_policy_updates': security_policy_updates
        }

    def export_attachments_to_file(self, export_file, name_list=None, device_type=None):
        """Export attachments to a file.  All object IDs will be translated to names.  Use
        a '.yml' extention to export as YAML and a '.json' extension to export as JSON.

        Args:
            export_file (str): The name of the export file

        """

        if name_list is None:
            name_list = []

        device_template_dict = self.device_templates.get_device_template_dict()

        attachments_list = []
        # Create a device config of the right type of things
        device_list = []
        if device_type in (None, 'controllers'):
            device_list = self.vmanage_device.get_device_config_list('controllers')
        if device_type in (None, 'vedges'):
            edge_list = self.vmanage_device.get_device_config_list('vedges')
            device_list = device_list + edge_list

        for device_config in device_list:
            if 'configStatusMessage' in device_config and device_config['configStatusMessage'] != 'In Sync':
                continue
            if 'template' in device_config:
                if device_config['template'] in device_template_dict:
                    template_id = device_template_dict[device_config['template']]['templateId']
                else:
                    raise Exception(f"Could not find ID for template {device_config['template']}")
                if name_list == [] or device_config.get('host-name') in name_list:
                    variable_dict = {}
                    template_input = self.device_templates.get_template_input(template_id,
                                                                              device_id_list=[device_config['uuid']])
                    data = template_input['data'][0]
                    for column in template_input['columns']:
                        variable_dict[column['variable']] = data[column['property']]
                    entry = {
                        'host_name': device_config.get('host-name'),
                        'device_type': device_config['deviceType'],
                        'uuid': device_config['chasisNumber'],
                        'system_ip': device_config['deviceIP'],
                        'site_id': device_config.get('site-id'),
                        'template': device_config['template'],
                        'variables': variable_dict
                    }
                    attachments_list.append(entry)

        attachment_export = {'vmanage_attachments': attachments_list}
        if export_file.endswith('.json'):
            with open(export_file, 'w') as outfile:
                json.dump(attachment_export, outfile, indent=4, sort_keys=False)
        elif export_file.endswith(('.yaml', 'yml')):
            with open(export_file, 'w') as outfile:
                yaml.dump(attachment_export, outfile, indent=4, sort_keys=False)
        else:
            raise Exception("File format not supported")
        return (len(attachments_list))

    def import_attachments_from_file(self, import_file, update=False, check_mode=False, name_list=None):
        """Import attachments from a file.  All object Names will be translated to IDs.

        Args:
            import_file (str): The name of the import file
            check_mode (bool): Try the import, but don't make changes (default: False)
            update (bool): Update existing templates (default: False)

        """

        template_data = {}
        # Read in the datafile
        if not os.path.exists(import_file):
            raise Exception(f"Cannot find file {import_file}")
        with open(import_file) as f:
            if import_file.endswith('.yaml') or import_file.endswith('.yml'):
                template_data = yaml.safe_load(f)
            else:
                template_data = json.load(f)

        imported_attachment_list = []
        if 'vmanage_attachments' in template_data:
            for attachment in template_data['vmanage_attachments']:
                if name_list is None or attachment['host_name'] in name_list:
                    imported_attachment_list.append(attachment)

        # Process the device templates
        result = self.template_data.import_attachment_list(imported_attachment_list,
                                                           check_mode=check_mode,
                                                           update=update)
        return result



import ipaddress

import click
from vmanage.api.device import Device
from vmanage.cli.show.print_utils import print_json


@click.command()
@click.argument('dev', required=False)
@click.option('--type',
              '-t',
              'device_type',
              required=False,
              default='all',
              type=click.Choice(['edge', 'control', 'all']),
              help="Device type [vedges, controllers]")
@click.option('--json/--no-json', default=False)
@click.pass_obj
def status(ctx, dev, device_type, json):  #pylint: disable=unused-argument
    """
    Show device status information
    """

    vmanage_device = Device(ctx.auth, ctx.host, ctx.port)
    # output = mn.get_control_connections_history(sysip)
    # vmanage_session = ctx.obj

    if dev:
        # Check to see if we were passed in a device IP address or a device name
        try:
            system_ip = ipaddress.ip_address(dev)
            device_dict = vmanage_device.get_device_status(system_ip)
        except ValueError:
            device_dict = vmanage_device.get_device_status(dev, key='host-name')

        if device_dict:
            print_json(device_dict)
        else:
            click.secho(f"Could not find device {dev}", err=True, fg='red')
    else:
        device_list = vmanage_device.get_device_status_list()
        if json:
            print_json(device_list)
        else:
            click.echo(
                f"{'Hostname':20} {'System IP':15} {'Model':15} {'Site':6} {'Status':9} {'BFD':>3} {'OMP':>3} {'CON':>3} {'Version':8} {'UUID':40} {'Serial'}"
            )
            for device_entry in device_list:
                if 'bfdSessionsUp' in device_entry:
                    bfd = device_entry['bfdSessionsUp']
                else:
                    bfd = ''
                if 'ompPeers' in device_entry:
                    omp = device_entry['ompPeers']
                else:
                    omp = ''
                if 'controlConnections' in device_entry:
                    control = device_entry['controlConnections']
                else:
                    control = ''
                click.echo(
                    f"{device_entry['host-name']:20} {device_entry['system-ip']:15} {device_entry['device-model']:15} {device_entry['site-id']:6} {device_entry['reachability']:9} {bfd:>3} {omp:>3} {control:>3} {device_entry['version']:8} {device_entry['uuid']:40} {device_entry['board-serial']}"
                )

    #     device_list = [device_ip]


@click.command()
@click.argument('dev', required=False)
@click.option('--type',
              '-t',
              'device_type',
              required=False,
              default='all',
              type=click.Choice(['edge', 'control', 'all']),
              help="Device type [vedges, controllers]")
@click.option('--json/--no-json', default=False)
@click.pass_obj
def config(ctx, dev, device_type, json):
    """
    Show device config information
    """
    vmanage_device = Device(ctx.auth, ctx.host, ctx.port)

    #pylint: disable=too-many-nested-blocks
    if dev:
        # Check to see if we were passed in a device IP address or a device name
        try:
            system_ip = ipaddress.ip_address(dev)
            device_dict = vmanage_device.get_device_status(system_ip)
        except ValueError:
            device_dict = vmanage_device.get_device_status(dev, key='host-name')

        if device_dict:
            if device_dict['device-type'] in ['vmanage', 'vbond', 'vsmart']:
                device_type = 'controllers'
            else:
                device_type = 'vedges'

            device_config = vmanage_device.get_device_config(device_type, device_dict['system-ip'])
            print_json(device_config)
        else:
            click.secho(f"Could not find device {dev}", err=True, fg='red')

    else:
        if not json:
            click.echo(
                f"{'Hostname':20} {'Device IP':15} {'Model':15} {'Site':6} {'State':9} {'Template':16} {'Status':7} {'Connection':10} {'Version':7}"
            )
        if device_type in ['all', 'control']:
            device_list = vmanage_device.get_device_config_list('controllers')

            if json:
                print_json(device_list)
            else:
                for device_entry in device_list:
                    if 'template' in device_entry:
                        template = device_entry['template']
                    else:
                        template = ''

                    device_name = device_entry['host-name'] if 'host-name' in device_entry else 'Unknown'
                    reachability = device_entry['reachability'] if 'reachability' in device_entry else 'Unknown'
                    site_id = device_entry['site-id'] if 'site-id' in device_entry else 'Unknown'
                    config_status_message = device_entry[
                        'configStatusMessage'] if 'configStatusMessage' in device_entry else 'Unknown'
                    vmanage_connection_state = device_entry[
                        'vmanageConnectionState'] if 'vmanageConnectionState' in device_entry else 'Unknown'
                    version = device_entry['version'] if 'version' in device_entry else 'Unknown'
                    click.echo(
                        f"{device_name:20} {device_entry['deviceIP']:15} {device_entry['deviceModel']:15} {site_id:6} {reachability:9} {template:16} {config_status_message:7} {vmanage_connection_state:10} {version:7}"
                    )

        if device_type in ['all', 'edge']:
            device_list = vmanage_device.get_device_config_list('vedges')
            if json:
                print_json(device_list)
            else:
                for device_entry in device_list:
                    if 'host-name' in device_entry:
                        if 'template' in device_entry:
                            template = device_entry['template']
                        else:
                            template = ''
                        device_name = device_entry['host-name'] if 'host-name' in device_entry else 'Unknown'
                        reachability = device_entry['reachability'] if 'reachability' in device_entry else 'Unknown'
                        site_id = device_entry['site-id'] if 'site-id' in device_entry else 'Unknown'
                        config_status_message = device_entry[
                            'configStatusMessage'] if 'configStatusMessage' in device_entry else 'Unknown'
                        vmanage_connection_state = device_entry[
                            'vmanageConnectionState'] if 'vmanageConnectionState' in device_entry else 'Unknown'
                        version = device_entry['version'] if 'version' in device_entry else 'Unknown'
                        click.echo(
                            f"{device_name:20} {device_entry['deviceIP']:15} {device_entry['deviceModel']:15} {site_id:6} {reachability:9} {template:16} {config_status_message:7} {vmanage_connection_state:10} {version:7}"
                        )


@click.group()
def device():
    """
    Show device information
    """


device.add_command(status)
device.add_command(config)



import click
from vmanage.api.certificate import Certificate


@click.command('install')
@click.option('--file', '-f', help="File containing the certificate to install.", required=True)
@click.pass_obj
def install(ctx, cert):
    """
    Install certificate
    """

    vmanage_certificate = Certificate(ctx.auth, ctx.host, ctx.port)
    click.echo("Installing certificate...")
    vmanage_certificate.install_device_cert(cert)
